package ru.other.company.loy.service.provider;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Objects;

import ru.crystals.pos.api.ext.loyal.dto.BonusAccrualResult;
import ru.crystals.pos.api.ext.loyal.dto.Discount;
import ru.crystals.pos.api.ext.loyal.dto.LoyaltyResult;
import ru.crystals.pos.api.ext.loyal.dto.Slip;
import ru.crystals.pos.api.ext.loyal.dto.SlipParagraph;
import ru.crystals.pos.api.ext.loyal.dto.SlipParagraphType;
import ru.crystals.pos.spi.receipt.Card;
import ru.crystals.pos.spi.receipt.LineItem;
import ru.crystals.pos.spi.receipt.Receipt;

/**
 * Стратегия поведения плагина при расчете скидок для чека продажи
 */
public class DoDiscountSaleStrategy {

    private ExampleLoyProvider plugin;
    private DiscountDistributor discountDistributor;

    public DoDiscountSaleStrategy(ExampleLoyProvider plugin) {
        this.plugin = plugin;
        discountDistributor = new DiscountDistributor(plugin);
    }

    /**
     * Применяет к чеку продажи техпроцесс расчета скидок.
     * @param receipt чек продажи
     * @return результат расчета скидок
     */
    public LoyaltyResult apply(Receipt receipt) {
        plugin.log.info("Inside {}#apply({})", this.getClass(), receipt);
        LoyaltyResult result = new LoyaltyResult();

        // Просто для примера - если есть "наша" карта, позволим списать с неё 3 бонуса как скидку
        String cardNumber = null;
        for(Card card : receipt.getCards()) {
            if(!Contstants.MY_CARD_ID.equals(card.getProcessingId())) {
                plugin.log.debug("Card \"{}\" (\"{}\") does not belong to \"{}\". Skipping it.", card.getCardNumber(), card.getProcessingId(), Contstants.NAME);
                continue;
            }
            // С версии 0.0.32 плагин карт может прицеплять к карте кастомные атрибуты.
            // Получим их здесь и используем зачем-то.
            // Следует отметить, плагин карт, этап поиска карты - единственное место, где допустимо
            // цеплять картам атрибуты сейчас.
            plugin.log.debug("Card attribute \"my.card.attribute.1\" = \"{}\"", card.getExtendedAttributes().get("my.card.attribute.1"));
            plugin.log.debug("Card attribute \"my.card.attribute.2\" = \"{}\"", card.getExtendedAttributes().get("my.card.attribute.2"));
            cardNumber = card.getCardNumber();
            if(card.getBonusBalance() != null && card.getBonusBalance().getSumToChargeOff() != null &&
                    card.getBonusBalance().getSumToChargeOff().compareTo(BigDecimal.ZERO) > 0) {
                // Если мы попали сюда, с карты списывались бонусы как скидка.
                // Сейчас плагины умеют предоставлять скидки только на позиции, а не на весь чек, поэтому размажем величину скидки по позициям.
                // Для упрощения, предоставим скидку на первую позицию, слежение за МРЦ и отрицательной стоимостью опущено.
                result.getDiscounts().add(new Discount(1, card.getCardNumber(), card.getBonusBalance().getSumToChargeOff(), Contstants.BONUS_DISCOUNT_ID));
            } else {
                // С этой карты не пытались списать бонусы ещё, заполним максимально разрешенную величину к списанию.
                // Это даст возможность списать их как скидка и учесть при следующем расчете (тогда мы попадём в верхний блок вместо сюда).
                result.getBonusWriteOffsLimits().put(card.getCardNumber(), calcMaxBonusWriteOffSum(receipt));
            }
        }
        result = discountDistributor.distribute(result, receipt);

        // Добавим несколько печатных слипов к расчету скидок
        result = addPrintSlips(result);
        // Добавим сообщение кассру, которое отображается после расчета скидок
        result.getMessagesToCashier().add("Выдайте клиету в подарок сдачу.");

        // Эти атрибуты будет сохранён в чеке и будут выгружаться в ERP. Атрибуты будут доступны плагину с момента добавления и на протяжении
        // всей жизни чека.
        // Хорошей практикой является именовать атрибуты, добавляя идентификатор своего плагина в качестве префикса.
        // Таким образом, будет меньше шансов, что два разных плагина непредусмотрено станут использовать один и тот же атрибут, перетирая его.
        result.getExtendedReceiptAttributesMap().put("ext.loy.processing.example.external.receipt.id", "123469877");
        result.getExtendedReceiptAttributesMap().put("ext.loy.processing.example.some.other.field", "whatever");

        // Мы можем уведомить кассу о том, что плагин лояльности начислил на карту бонусные баллы.
        // Это необходимо, чтобы отобразить событие начисление бонусов на карту на сервере в Опердне или выгрузке в ERP.
        // При отмене расчета скидок уведомление будет удалено кассой автоматически.
        if(cardNumber != null) {
            // Уведомляем кассу о начислении бонусов на карту.
            result.getBonusAccrualResults().put(cardNumber, new BonusAccrualResult("123456-8789", calcMaxBonusAccrualSum(receipt)));
        }

        plugin.log.info("leaving {}#apply() result {}", this.getClass(), result);
        return result;
    }

    /**
     * Добавляет печатные слипы к результату расчета скидок в внешней системе.
     * @param result результат расчета скидок требуется добавить печатные слипы которому.
     * @return тот же экземпляр результата расчета скидок, но с добавленными печатными слипами.
     */
    private LoyaltyResult addPrintSlips(LoyaltyResult result) {
        Slip slip1 = new Slip();
        slip1.getParagraphs().add(
                new SlipParagraph(SlipParagraphType.TEXT, "Реклама на чеке:\nКупите пряник малиновый сладкий и получите ничего в подарок!")
        );
        result.getPrintDocuments().add(slip1);

        Slip slip2 = new Slip();
        slip2.getParagraphs().add(
                new SlipParagraph(SlipParagraphType.TEXT, "Спасибо, что воспользовались нашим плагином лояльности.\nЭто второй печатный слип")
        );
        // Это строка в слипе будет распечатана как баркод.
        slip2.getParagraphs().add(new SlipParagraph(SlipParagraphType.BARCODE, "123469879792545454"));
        result.getPrintDocuments().add(slip2);
        return result;
    }

    /**
     * Расчет максимально разрешенной величины к списанию
     * @param receipt чек
     * @return максимально разрешенную величину к списанию
     */
    private BigDecimal calcMaxBonusWriteOffSum(Receipt receipt) {
        // расчитываем попозиционно
        return receipt.getLineItems().stream()
                .map(item -> calcSum(item, getPayBonusAllowedQuantity(item, receipt))).reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    /**
     * Расчет максимально разрешенной величины к начислению
     * @param receipt чек
     * @return максимально разрешенную величину к начислению
     */
    private BigDecimal calcMaxBonusAccrualSum(Receipt receipt) {
        // расчитываем попозиционно
        return receipt.getLineItems().stream()
                .map(item -> calcSum(item, getAccrueBonusAllowedQuantity(item, receipt))).reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    private long getPayBonusAllowedQuantity(LineItem position, Receipt receipt) {
        //  проверяем разрешено ли вообще списание бонусов на позицию
        if (position == null || !position.isPayBonusAllowed()) {
            return 0L;
        }
        // вычисляем допустимое к списанию кол-во, исходя из запрещающих акций
        long unAllowedQuantity = position.getAppliedDiscounts().stream()
                .filter(ad -> receipt.getAppliedAdvertisingActions().stream()
                        .anyMatch(aa -> !aa.isPayBonusAllowed() && Objects.equals(aa.getGuid(), ad.getAdvertisingActionGuid())))
                .mapToLong(ad -> plugin.currencyHandler.convertQuantity(ad.getQuantity())).max().orElse(0L);
        return position.getQuantity() - unAllowedQuantity;
    }

    private long getAccrueBonusAllowedQuantity(LineItem position, Receipt receipt) {
        //  проверяем разрешено ли вообще начисление бонусов на позицию
        if (position == null || !position.isAccrueBonusAllowed()) {
            return 0L;
        }
        // вычисляем допустимое к начислению кол-во, исходя из запрещающих акций
        long unAllowedQuantity = position.getAppliedDiscounts().stream()
                .filter(ad -> receipt.getAppliedAdvertisingActions().stream()
                        .anyMatch(aa -> !aa.isAccrueBonusAllowed() && Objects.equals(aa.getGuid(), ad.getAdvertisingActionGuid())))
                .mapToLong(ad -> plugin.currencyHandler.convertQuantity(ad.getQuantity())).max().orElse(0L);
        return position.getQuantity() - unAllowedQuantity;
    }

    private BigDecimal calcSum(LineItem position, long quantity) {
        BigDecimal endPrice = position.getSum().divide(plugin.currencyHandler.convertQuantity(position.getQuantity()), 2, RoundingMode.HALF_UP);
        return plugin.currencyHandler.getPositionSum(endPrice, quantity);
    }
}
