package ru.crystals.pos.api.plugin;

import java.math.BigDecimal;
import ru.crystals.pos.api.plugin.card.BonusOperationResponse;
import ru.crystals.pos.api.plugin.card.BonusOperationStatus;
import ru.crystals.pos.api.plugin.card.BonusWriteOffOperationResponse;
import ru.crystals.pos.api.plugin.card.CardInfo;
import ru.crystals.pos.api.plugin.card.CardSearchResponse;
import ru.crystals.pos.api.plugin.card.CardSearchResponseStatus;
import ru.crystals.pos.spi.plugin.card.BonusWriteOffCallback;
import ru.crystals.pos.spi.plugin.card.CardSearchCallback;
import ru.crystals.pos.spi.plugin.card.CardSearchRequest;
import ru.crystals.pos.spi.receipt.Card;
import ru.crystals.pos.spi.receipt.Receipt;

/**
 * Интерфейс плагина карт (процессинга карт/купонов лояльности).
 *
 */
public interface CardPlugin {

    /**
     * Поиск информации по карте на основе предоставленных данных для поиска.
     *
     * @param request исходные данные для поиска карты
     * @return не {@code null}; при возникновении нештатных ситуаций следует возвращать объект с не-{@link CardSearchResponseStatus#OK позитивным}
     * {@link CardSearchResponse#getResponseStatus() статусом} обработки запроса
     */
    CardSearchResponse searchCard(CardSearchRequest request);

    /**
     * Поиск информации по карте на основе предоставленных данных для поиска.<br>
     * В качестве данных для поиска в метод могут быть переданы email или номер телефона.
     * Метод не асинхронный, использует калбек для передачи результата, управление возвращает при первой же возможности.
     * В случае, если метод имеет реализацию (использует калбек), требуется чтобы {@link #searchCardByMobileNumber(String, CardSearchCallback)} не был реализован.
     * В любом случае для корректной работы кассы требуется, чтобы один из этих методов был реализован!
     * @since 0.0.43
     * @param request исходные данные для поиска карты
     * @param callback калбек, который вызовется при завершении операции
     */
    default void searchCardAsync(CardSearchRequest request, CardSearchCallback callback) {
        callback.onCardSearchCompleted(new CardSearchResponse(CardSearchResponseStatus.UNKNOWN_CARD));
    }

    /**
     * Выполняет поиск карты по номеру телефона.<br>
     * Метод не асинхронный, возвращает результат выполнения через метод обраного вызова, а управление при первой же возможности.
     * @deprecated метод устарел и не рекомендуется к использованию. Используйте метод {@link #searchCardAsync(CardSearchRequest, CardSearchCallback)}
     * В случае, если метод имеет реализацию (использует калбек), требуется чтобы {@link #searchCardAsync(CardSearchRequest, CardSearchCallback)} не был реализован.
     * Для корректной работы кассы требуется, чтобы один из этих методов был реализован!
     * @param mobileNumber номер мобильного телефона, по которому необходимо найти карту.
     * @param callback калбек, который вызовется по завершению операции
     */
    @Deprecated
    default void searchCardByMobileNumber(String mobileNumber, CardSearchCallback callback) {
        callback.onCardSearchCompleted(new CardSearchResponse(CardSearchResponseStatus.UNKNOWN_CARD));
    }

    /**
     * Производит списание бонусов с указанной карты. Этот метод синхронный и не возвращает управление, пока списание не будет выполнено.
     * Также в этом методе запрещено пользоваться {@link ru.crystals.pos.spi.ui.UIForms}, вызов методов этого класса не будет иметь эффекта.
     * @see #writeOffAsync(Card, BigDecimal, Receipt, BonusWriteOffCallback)
     * @param bonusCard бонусная карта, с баланса которой собираемся списать бонусы
     * @param bonusesToWriteOff количество бонусов для списания
     * @param receipt чек, в котором производится данное списание бонусов
     * @return структуру {@link BonusWriteOffOperationResponse}, описывающую результат выполнения операции или null, если данный
     * метод плагином не используется для списания и следует вызывать метод {@link #writeOffAsync(Card, BigDecimal, Receipt, BonusWriteOffCallback)}.
     * @see #writeOffAsync(Card, BigDecimal, Receipt, BonusWriteOffCallback)
     */
    default BonusWriteOffOperationResponse writeOff(Card bonusCard, BigDecimal bonusesToWriteOff, Receipt receipt) {
        return new BonusWriteOffOperationResponse(BonusOperationStatus.ERROR, null, null);
    }

    /**
     * Производит списание бонусов с указанной карты. Этот метод не асинхронный, возвращает результат выполнения через метод обратного вызова, а управление
     * при первой же возможности.
     * Результат выполнения метода передаётся через метод обратного вызова {@link BonusWriteOffCallback#onBonusWriteOffCompleted(BonusWriteOffOperationResponse)}.
     * В отличии отметода {@link #writeOff(Card, BigDecimal, Receipt)}, в данном методе разрешено использовать {@link ru.crystals.pos.spi.ui.UIForms}.<br>
     * Если плагину требуется манипулировать UI при списании бонусов, он не должен реализовывать логику в методе {@link #writeOff(Card, BigDecimal, Receipt)}, а должен
     * реализовывать её здесь. Методу {@link #writeOff(Card, BigDecimal, Receipt)} в таком случае требуется возвращать null.<br>
     * Касса выбирает какой метод вызывать по следующему алгоритму:
     * <ol>
     *  <li>Вызвать метод {@link #writeOff(Card, BigDecimal, Receipt)}</li>
     *  <li>Если метод вернул null, вызывать метод асинхронного списания.</li>
     *  <li>В противном случае не вызывать метод асинхронного списания.</li>
     * </ol>
     * @since 0.0.35
     * @param bonusCard бонусная карта, требуется списать бонусы с которой
     * @param amount величина к списанию
     * @param receipt чек, в рамках которого производится списание бонусов
     * @param callback структура, содержащая метод обратного вызова, через который кассе будет передан результат списания бонусов.
     * @see #writeOff(Card, BigDecimal, Receipt)
     */
    default void writeOffAsync(Card bonusCard, BigDecimal amount, Receipt receipt, BonusWriteOffCallback callback) {
        // Не проверяем на null специально чтобы словить здесь NPE, потому что это будет указывать
        // на большую ошибку в кассе - каллбек не должен быть null!

        // А ответим null для того, чтобы касса могла знать, этот метод не реализован и делает ничего.
        callback.onBonusWriteOffCompleted(null);
    }
    /**
     * Отмена списания бонусов.
     *
     * @param txId {@link BonusWriteOffOperationResponse#getTxId() идентификатор} транзакции списания бонусов, что надо "откатить"
     * @return не {@code null} - результат выполнения операции
     */
    default BonusOperationResponse rollback(String txId) {
        return new BonusOperationResponse(BonusOperationStatus.ERROR, null);
    }

    /**
     * Возвращает информацию о карте и её держателе.
     * @param request номер карты, по которой следует получить информацию.
     * @return информация о карте или иной результат, если карта не найдена.
     */
    default CardInfo getCardInfo(CardSearchRequest request) {
        return new CardInfo(CardSearchResponseStatus.UNKNOWN_CARD, null);
    }
}
