package ru.crystals.pos.api.plugin;

import ru.crystals.pos.api.receipt.MerchandiseEntity;
import ru.crystals.pos.spi.plugin.FiscalizationListener;
import ru.crystals.pos.spi.plugin.ReconciliationReportMaker;
import ru.crystals.pos.spi.plugin.goods.AddForRefundRequest;
import ru.crystals.pos.spi.plugin.goods.AddForSaleRequest;
import ru.crystals.pos.spi.plugin.goods.CancelDeniedException;
import ru.crystals.pos.spi.plugin.goods.RemoveFromSaleRequest;

/**
 * Интерфейс плагина товара.
 */
public interface GoodsPlugin extends FiscalizationListener, ReconciliationReportMaker {

    /**
     * Запрос на добавление товара в чек для продажи.
     * <p>
     * Касса вызывает этот метод, когда необходимо добавить товар в чек.<br>
     * Товар содержится в аргументе {@code addForSaleRequest}.
     * <p>
     * Результат добавления/не добавления товара возвращается в кассу обратными
     * (<i>callback</i>) вызовами содержащимися в аргументе
     * {@code addForSaleRequest}.
     *
     * @param addForSaleRequest
     *            необходимые для добавления товара данные и реализация callback интерфейса
     *
     * @see AddForSaleRequest
     */
    void addForSale(AddForSaleRequest addForSaleRequest);


    /**
     * Запрос на добавление товара в чек произвольного возврата. Чек произвольного возврата образуется, когда
     * пользователь желает вернуть один или несоклько товаров не по чеку продажи, по которому они были куплены.
     * Таким образом, ссылки на оригинальный чек продажи у такого чека не будет.
     * @param request тело запроса на добавление товара, должен содержать товар и метод, который плагин обязан вызвать для
     *                передачи результата кассе.
     * @since 0.0.34
     */
    default void addForRefund(AddForRefundRequest request) {
        // Специально не выполняем проверок на null, потому что он здесь - недопустимая ситуация,
        // посему должно падать.
        try {
            request.getCallback().notCompleted();
        } catch(CancelDeniedException cdex) {
            // Нечего обрабатывать
        }
    }

    /**
     * Запрос на удаление товарной позиции из чека продажи.
     * <p>
     * Касса вызывает метод, когда необходимо удалить добавленную позицию из чека, дважды.<br>
     * Первый раз управление передается плагину для проверки возможности удаления.<br>
     * Далее, если удаление возможно, на кассе выводится диалог: "Удалить позицию из чека?" (Да | Нет)<br>
     * После положительного выбора управление передается плагину для выполнения необходимых действий
     * с позицией перед удалением из чека.
     * Под удалением подразумевается, что плагин выполнит все необходимые ему действия.<br>
     * Удаление записи из БД касса выполняет после получения второго положительного ответа в callback, т.е.:<br>
     * {@link ru.crystals.pos.spi.plugin.goods.RemoveFromSaleCallback#completed(boolean)}
     *
     * @param request содержит удаляемую позицию, чек и реализацию callback интерфейса
     */
    void removeFromSale(RemoveFromSaleRequest request);

    /**
     * Запрос на поиск товара по штрихкоду.<p>
     * Касса вызывает этот метод, когда был просканирован штрихкод и касса находится в режиме добавления товаров в чек.<br>
     * Если плагин распознал шрихкод как товар, продажу которого он осуществляет, то необходимо вернуть новую сущность
     * {@link MerchandiseEntity}, иначе {@code null}.<br>
     * В режиме продажи, касса передаст управление плагину вызовом метода {@link #addForSale(AddForSaleRequest)},
     * найденный товар можно будет получить методом {@link AddForSaleRequest#getMerchandise()}.
     *
     * @param barcode штрихкод
     * @return сущность товара или {@code null}
     */
    MerchandiseEntity findByBarcode(String barcode);

}
