package ru.crystals.pos.spi.receipt;

import java.util.Collections;
import java.util.Map;
import ru.crystals.pos.spi.plugin.card.CardSearchEventSource;

/**
 * Интерфейс для получения информации о карте/купоне.
 */
public interface Card {

    /**
     * Получить уникальный идентификатор карты - идентификатор достаточный для получения полной информации по данной карте
     * <p>
     * NOTE: в большинстве случаев будет совпадать с {@link #getCardNumber()} номером} карты. Но возможны и исключения:
     * например, это может быть NFC-метка - и информацию по карте (включая ее номер) можно получить только сделав запрос в
     * процессинговый центр (передав туда эту NFC-метку) владельца данной карточной системы.
     *
     * @return идентификатор карты
     */
    String getId();

    /**
     * Получить номер карты
     *
     * @return {@code null}, если номер карты не известен
     */
    String getCardNumber();

    /**
     * Получить тип карты
     * 
     * @return тип карты
     * @see CardType
     */
    CardType getCardType();

    /**
     * Получить владельца карты
     *
     * @return {@code null}, если карта не персонализированная или владелец неизвестен
     * @see CardHolder
     */
    CardHolder getCardHolder();

    /**
     * Получить идентификатор процессинга, которому "принадлежит" карта
     *
     * @return {@code null}, если идентификатор процессинга неизвестен
     */
    String getProcessingId();

    /**
     * Получить бонусный баланс карты
     *
     * @return {@code null}, если у карты нету бонусного баланса
     * @see BonusBalance
     */
    BonusBalance getBonusBalance();

    /**
     * Получить статус данной карты/купона
     *
     * @return {@code null} будет рассматриваться как {@link CardStatus#ACTIVE}
     * @see CardStatus
     */
    CardStatus getCardStatus();

    /**
     * Возвращает способ, которым карта была считана.
     * @since 0.0.31
     * @return способ, которым карта была считана.
     */
    CardSearchEventSource getInputType();

    /**
     * Получает мапу атрибутов, которые были добавлены к карте.<br>
     * Позволяет добавлять к карте любые дополнительные параметры, которые затем можно использовать где-нибудь в другом месте.
     * @return мапу атрибутов, которые были добавлены к карте или пустую мапу, если таких нет.
     * @since 0.0.32
     */
    default Map<String, String> getExtendedAttributes() {
        return Collections.EMPTY_MAP;
    }
}
